from sqlalchemy import Column, create_engine, func, ForeignKey
from sqlalchemy.dialects.postgresql import UUID, VARCHAR, TIMESTAMP, SMALLINT, JSONB, TEXT, BOOLEAN, BIGINT
from sqlalchemy.ext.declarative import declarative_base
from uuid import uuid4
from sqlalchemy.types import TypeDecorator
from sqlalchemy import DateTime as SdateTime
from datetime import datetime, timezone
import pytz

#Mexico city time zone
mx_cty_tz = pytz.timezone('America/Mexico_City')

#Function to get timezone aware current time
now = lambda : datetime.utcnow().replace(tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)

Base = declarative_base()

class DateTime(TypeDecorator):
    impl = SdateTime
    def process_bind_param(self, value, engine):
        if value is None:
            return value
        return value.replace(tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)
    def process_result_value(self, value, engine):  
        return value

class Workers(Base):
    __tablename__ = "workers"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    persona = Column(VARCHAR(12))
    name = Column(VARCHAR(50))
    last_name = Column(VARCHAR(50))
    email = Column(VARCHAR(150))
    username = Column(VARCHAR(30))
    status = Column(SMALLINT)
    logged_at = Column(TIMESTAMP)
    session_deadline = Column(TIMESTAMP)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)
    session_id = Column(VARCHAR(50))

class WorkerHistory(Base):
    __tablename__ = "worker_history"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=False)
    status_code = Column(VARCHAR(20))
    status = Column(VARCHAR(20))
    reason = Column(TEXT)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)

class Tasks(Base):
    __tablename__ = "tasks"

    uuid = Column(UUID(as_uuid=True),primary_key=True)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=True)
    name = Column(VARCHAR(150))
    data = Column(JSONB)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    priority = Column(SMALLINT)
    deadline = Column(TIMESTAMP)
    requested_at = Column(TIMESTAMP)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)

class TaskHistory(Base):
    __tablename__ = "task_history"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    task_uuid = Column(UUID(as_uuid=True),ForeignKey(Tasks.uuid), nullable=False)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=False)
    data = Column(JSONB)
    reason = Column(TEXT)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)

class Noticia(Base):
    __tablename__ = "noticia"

    noticia_id = Column(BIGINT,autoincrement=True,primary_key=True)
    titulo = Column(VARCHAR(128), default='',nullable=False)
    resumen = Column(VARCHAR(1024), default='',nullable=False)
    fecha_inicio_publicacion = Column(TIMESTAMP, default=now, nullable=True)
    fecha_fin_publicacion = Column(TIMESTAMP)
    activa = Column(BOOLEAN, default=True)
    imagen = Column(VARCHAR(2048))



# if __name__ == "__main__":
#     engine = create_engine("postgresql://cloud:af9c7KHRugnCK7E@35.223.124.237:5432/task_manager_test")
#     Base.metadata.create_all(bind=engine)